/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 *                                                                         *
 *  This module (C) 1994, 1995, 1996 Robert L. Peckham (Rip)               *
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"



void do_break( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *room = ch->in_room;
    int direction = -1;

    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char(
	  "You don't have authorization to build in this zone.\n\r", ch );
	return;
    }

    if ( !str_cmp( argument, "all" ) )
    {
	for ( direction = 0; direction <= 5; direction++ )
	{
	    if ( room->exit[direction] == NULL )
		continue;

	    if ( room->exit[direction]->keyword[0] == '\0'
		&& room->exit[direction]->description[0] == '\0' )
	    {
		free_exit( room->exit[direction] );
		room->exit[direction] = NULL;
	    }
	    else
	    {
		room->exit[direction]->to_room = NULL;
		room->exit[direction]->vnum = 0;
		room->exit[direction]->exit_flags = 0;
		room->exit[direction]->key = 0;
	    }
	}

	send_to_char("All exits broken.\n\r", ch );
	return;
    }

    direction = get_direction( argument );

    if ( direction == -1 )
    {
	send_to_char("Syntax:   break <direction>  (north/east, etc.  or all).\n\r", ch );
	return;
    }

    if ( room->exit[direction] == NULL )
    {
	send_to_char("No exit in that direction.\n\r", ch );
	return;
    }

    if ( room->exit[direction]->keyword[0] == '\0'
    && room->exit[direction]->description[0] == '\0' )
    {
	free_exit( room->exit[direction] );
	room->exit[direction] = NULL;
    }
    else
    {
	room->exit[direction]->to_room = NULL;
	room->exit[direction]->vnum = 0;
	room->exit[direction]->exit_flags = 0;
	room->exit[direction]->key = 0;
    }

    send_to_char("Exit removed.\n\r", ch );
    return;
}


void do_collaps( CHAR_DATA *ch, char *argument )
{
    send_to_char("You need to type collapse all the way out.\n\r", ch );
    return;
}

void do_collapse( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char("You don't have authorization to build in this zone.\n\r", ch );
	return;
    }

    if ( ch->in_room->vnum == ROOM_VNUM_CHAT )
    {
	send_to_char("You can't collapse the chat room.\n\r", ch );
	return;
    }

    act("$n collapses the room.", ch, NULL, NULL,
	NULL, NULL, TO_ROOM, SENSE_SIXTH );

    free_room( ch->in_room );

    send_to_char("Room deleted.\n\r", ch );
    return;
}

void do_connect( CHAR_DATA *ch, char *argument )
{
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    extern sh_int rev_dir[];
    int vnum;
    int direction = -1;
    bool fOneway = FALSE;
    ROOM_INDEX_DATA *pRoomIndex;
    EXIT_DATA *pexit = NULL;

    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char(
	  "You do not have authorization to build in this zone.\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
	send_to_char(" Syntax:  connect <direction> <vnum> [oneway]\n\r", ch );
	send_to_char(" [oneway] is optional parameter.  Exits are two way\n\r", ch );
	send_to_char(" by default.  Anything other than 'oneway' in that\n\r", ch );
	send_to_char(" field will be ignored.\n\r", ch );
	return;
    }

    if ( !is_number( arg2 ) )
    {
	send_to_char("Target room vnum must be numeric.\n\r", ch );
	return;
    }
    vnum = atoi( arg2 );

    direction = get_direction( arg1 );

    if ( direction == -1 )
    {
	do_connect( ch, "" );
	return;
    }

    if ( !str_cmp( argument, "oneway" ) || !str_cmp( argument, "one" ) )
	fOneway = TRUE;

    if ( (pRoomIndex = get_room_index( vnum )) == NULL )
    {
	send_to_char("Target room number does not exist.\n\r", ch );
	return;
    }

    if ( (pexit = ch->in_room->exit[direction]) == NULL )
    {
	pexit = (EXIT_DATA *) alloc_perm( sizeof(*pexit) );
	pexit->to_room = pRoomIndex;
	pexit->description = str_dup( "\0" );
	pexit->keyword = str_dup( "\0" );
	pexit->exit_flags = 0;
	pexit->key = 0;
	pexit->vnum = vnum;
	ch->in_room->exit[direction] = pexit;
    }
    else
    {
	if ( pexit->to_room != NULL )
	{
	    send_to_char("Exit already exists.\n\r", ch );
	    return;
	}

	pexit->to_room = pRoomIndex;
	pexit->vnum = vnum;
    }

    if ( !fOneway )
    {
	if ( (pexit = pRoomIndex->exit[rev_dir[direction]]) == NULL )
	{
	    pexit = (EXIT_DATA *) alloc_perm( sizeof(*pexit) );
	    pexit->to_room = ch->in_room;
	    pexit->description = str_dup( "\0" );
	    pexit->keyword = str_dup( "\0" );
	    pexit->exit_flags = 0;
	    pexit->key = 0;
	    pexit->vnum = ch->in_room->vnum;
	    pRoomIndex->exit[rev_dir[direction]] = pexit;
	}
	else
	{
	    if ( pexit->to_room != NULL )
	    {
		send_to_char("Reverse exit already exists.  Forward entrance only created.\n\r", ch );
		return;
	    }

	    pexit->to_room = ch->in_room;
	    pexit->vnum = ch->in_room->vnum;
	}
    }

    send_to_char("Exit connected.\n\r", ch );
    char_from_room( ch );
    char_to_room( ch, pRoomIndex, FALSE );
    do_look( ch, "auto" );
    return;
}


ROOM_INDEX_DATA *new_room( AREA_DATA *pArea, int vnum, char *room_name )
{
  ROOM_INDEX_DATA *pRoomIndex;
  extern ROOM_INDEX_DATA *room_free;
  extern ROOM_INDEX_DATA *room_index_hash[MAX_KEY_HASH];
  extern int top_room;
  int iHash;

  if ( room_free == NULL ) {
    pRoomIndex = (ROOM_INDEX_DATA *) alloc_perm( sizeof(*pRoomIndex));
  } else {
    pRoomIndex = room_free;
    room_free = pRoomIndex->next;
  }

  if ( pRoomIndex == NULL ) {
    log_string( "Error opening new room." );
    return NULL;
  }

  pRoomIndex->people = NULL;
  pRoomIndex->contents = NULL;
  pRoomIndex->extra_descr = NULL;
  pRoomIndex->flora = NULL;
  pRoomIndex->area = pArea;
  pRoomIndex->vnum = vnum;

  if ( room_name == NULL || room_name[0] == '\0' )
    pRoomIndex->name = str_dup( "New Room." );
  else
    pRoomIndex->name = str_dup( room_name );

  pRoomIndex->description = str_dup( "Empty description.\n" );
  pRoomIndex->room_flags = 0;
  set_sector( pRoomIndex, sector_lookup( "city", NULL ) );
  pRoomIndex->move_cost = 0;
  pRoomIndex->move_delay = 0;
  pRoomIndex->light = 0;
  pRoomIndex->light_mod = 0;
  pRoomIndex->light_filter = 0;
  pRoomIndex->size = 0;
  pRoomIndex->spec_fun_name = str_dup( "\0" );
  pRoomIndex->spec_fun = NULL;

  iHash = vnum % MAX_KEY_HASH;
  pRoomIndex->next = room_index_hash[iHash];
  room_index_hash[iHash] = pRoomIndex;
  pRoomIndex->zone_next = NULL;
  top_room++;

  if ( pArea->room_first == NULL )
  {
    pRoomIndex->area->room_first = pRoomIndex;
    pRoomIndex->area->room_last = pRoomIndex;
  }
  else
  {
    pRoomIndex->area->room_last->zone_next = pRoomIndex;
    pRoomIndex->area->room_last = pRoomIndex;
  }

  return pRoomIndex;
}


void do_dig( CHAR_DATA *ch, char *argument )
{
  extern sh_int rev_dir[];
  int door;
  int direction;
  int vnum;
  bool fEonly = FALSE;
  char arg1 [MAX_INPUT_LENGTH];
  char arg2 [MAX_INPUT_LENGTH];
  char buf [MAX_STRING_LENGTH];
  ROOM_INDEX_DATA *pRoomIndex;
  EXIT_DATA *pexit = NULL;

  if ( IS_NPC( ch ) )
    return;

  if ( !can_build( ch, ch->in_room->area ) ) {
    send_to_char(
      "You do not have authorization to build in this zone.\n\r", ch );
    return;
  }

  argument = one_argument( argument, arg1 );

  if ( arg1[0] != '\0' && !str_cmp( arg1, "list" ) ) {
    int col = 0;

    sprintf( buf, "Room vnums in use for %s.\n\r", ch->in_room->area->name );
    send_to_char( buf, ch );

    for ( pRoomIndex = ch->in_room->area->room_first;
      pRoomIndex != NULL; pRoomIndex = pRoomIndex->zone_next ) {

      sprintf( buf, " %6d", pRoomIndex->vnum );
      send_to_char( buf, ch );
      if ( ( ++col % 10 ) == 0 )
        send_to_char( "\n\r", ch );
    }

    if ( col % 10 != 0 )
      send_to_char( "\n\r", ch );

    return;
  }

  argument = one_argument( argument, arg2 );

  if ( arg1[0] == '\0' || arg2[0] == '\0' ) {
    send_to_char(" Syntax:  dig new <vnum> [room name]\n\r", ch );
    send_to_char("          dig (north/south/east etc) <vnum> [room name]\n\r",
      ch );
    send_to_char("          dig list\n\r", ch );
    return;
  }

  if ( !is_number( arg2 ) ) {
    send_to_char("Vnum must be numeric.\n\r", ch );
    return;
  }
  vnum = atoi( arg2 );

/*  add vnum checking    */
  if ( get_room_index( vnum ) != NULL ) {
    send_to_char("Vnums can not be duplicated.\n\r", ch );
    return;
  }

  if ( vnum < ch->in_room->area->vnum_start
    || vnum > ch->in_room->area->vnum_final ) {
    send_to_char("Vnum out of range.\n\r", ch );
    sprintf( buf, "Vnum must be in range %d to %d.\n\r",
      ch->in_room->area->vnum_start,
      ch->in_room->area->vnum_final );
    send_to_char( buf, ch );
    return;
  }

  direction = 10;

  if ( !str_cmp( arg1, "new" ) )
    direction = -1;

  direction = get_direction( arg1 );

  if ( direction > 5 ) {
    do_dig( ch, "" );
    return;
  }

  if ( direction != -1 && ((pexit = ch->in_room->exit[direction]) != NULL)) {
    if ( pexit->to_room != NULL ) {
      send_to_char("There is already an exit in that direction.\n\r", ch );
      return;
    }
    fEonly = TRUE;
  }

  if ( ( pRoomIndex = new_room( ch->in_room->area, vnum, argument ) ) == NULL )
    return;

  for ( door = 0; door <= 5; door++ ) {
    pRoomIndex->exit[door] = NULL;
  }

  if ( direction != -1 ) {
    if (fEonly) {
      pexit->to_room = pRoomIndex;
    } else {
/****fixmefixme****/
      pexit = (EXIT_DATA *) alloc_mem( sizeof(*pexit) );
      pexit->to_room = pRoomIndex;
      pexit->description = str_dup( "\0" );
      pexit->keyword = str_dup( "\0" );
      pexit->exit_flags = 0;
      pexit->key = 0;
      pexit->vnum = vnum;
      ch->in_room->exit[direction] = pexit;
    }

    pexit = (EXIT_DATA *) alloc_perm( sizeof(*pexit) );
    pexit->to_room = ch->in_room;
    pexit->description = str_dup( "\0" );
    pexit->keyword = str_dup( "\0" );
    pexit->exit_flags = 0;
    pexit->key = 0;
    pexit->vnum = ch->in_room->vnum;
    pRoomIndex->exit[rev_dir[direction]] = pexit;
  }

  send_to_char("Room created.\n\r", ch );
  char_from_room( ch );
  char_to_room( ch, pRoomIndex, FALSE );
  do_look( ch, "auto" );
  return;
}


void do_exset( CHAR_DATA *ch, char *argument )
{
    char direction [MAX_INPUT_LENGTH];
    char command [MAX_INPUT_LENGTH];
    char buf [MAX_STRING_LENGTH];
    EXIT_DATA *pexit;
    extern sh_int rev_dir[];
    int door = -1;

    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char("You don't have authorization to build in this zone.\n\r", ch );
	return;
    }

    argument = one_argument( argument, direction );
    argument = one_argument( argument, command );

    if ( direction[0] == '\0' || command[0] == '\0' )
    {
	send_to_char("Syntax:\n\r", ch );
	send_to_char(" exset <direction> <command> <argument>\n\r", ch );
	send_to_char("\n\r", ch );
	send_to_char("Where argument is one of the following:\n\r", ch );
	send_to_char("desc cldesc key door/flags keyword lock/diff\n\r", ch );
	return;
    }

    door = get_direction( direction );

    if ( door == -1 )
    {
	send_to_char("Invalid direction.\n\r", ch );
	return;
    }

    if ( (pexit = ch->in_room->exit[door]) == NULL )
    {
	send_to_char("No exit data in that direction.  Creating entry.\n\r", ch );

	pexit = (EXIT_DATA *) alloc_perm( sizeof( *pexit ) );
	pexit->to_room = NULL;
	pexit->description = str_dup( "\0" );
	pexit->keyword = str_dup( "\0" );
	pexit->exit_flags = 0;
	pexit->key = 0;
	pexit->vnum = -1;
	pexit->difficulty = 5;
	ch->in_room->exit[door] = pexit;
    }

    if ( !str_cmp( command, "cldesc" ) )
    {
	buf[0] = '\0';

	free_string( pexit->description );
	pexit->description = str_dup( buf );

	send_to_char("Exit description cleared.\n\r", ch );
	return;
    }

    if ( !str_cmp( command, "desc" ) )
    {
	strcpy( buf, pexit->description );

	if ( strlen( buf ) + strlen( argument ) >= MAX_STRING_LENGTH -4 )
	{
	    send_to_char("Exit description too long, text not added.\n\r", ch );
	    return;
	}

	if ( buf[0] == '\0' )
	    strcpy( buf, argument );
	else
	    strcat( buf, argument );
	strcat( buf, "\n\r" );
	free_string( pexit->description );
	pexit->description = str_dup( buf );

	send_to_char("Text added to exit description.\n\r", ch );
	return;
    }

    if ( !str_cmp( command, "door" ) || !str_cmp( command, "flags" ) )
    {
	if (pexit->to_room == NULL)
	{
	    send_to_char("No exit, no door..\n\r", ch );
	    return;
	}

	if ( argument[0] == '\0' || !is_number( argument ) )
	{
	    send_to_char("door status must be numeric\n\r", ch );
	    return;
	}

	pexit->exit_flags = atoi( argument );

	if ( pexit->to_room->exit[rev_dir[door]] != NULL
	&& pexit->to_room->exit[rev_dir[door]]->to_room == ch->in_room )
	{
	    pexit->to_room->exit[rev_dir[door]]->exit_flags = atoi( argument );
	}

	return;
    }

    if ( !str_cmp( command, "key" ) )
    {
	if (pexit->to_room == NULL)
	{
	    send_to_char("No exit, no door..\n\r", ch );
	    return;
	}

	if (argument[0] == '\0' || !is_number( argument ) )
	{
	    send_to_char("key vnum must be numeric\n\r", ch );
	    return;
	}

	pexit->key = atoi( argument );

	return;
    }

    if ( !str_cmp( command, "keyword" ) )
    {
	if ( argument == NULL )
	{
	    send_to_char("Set what as keyword?\n\r", ch );
	    return;
	}
	free_string( pexit->keyword );
	pexit->keyword = str_dup( argument );
	return;
    }

    if ( !str_cmp( command, "lock" ) || !str_cmp( command, "diff" ) )
    {
	int level;

	if ( argument[0] == '\0' || !is_number( argument ) )
	{
	    send_to_char("Argument must be numeric.\n\r", ch );
	    return;
	}

	level = atoi( argument );

	if ( level < 0 || level > 95 )
	{
	    send_to_char("Lock level range is 0 to 95.\n\r", ch );
	    return;
	}

	pexit->difficulty = level;
	return;
    }

    do_exset( ch, "" );

    return;
}


void do_rdesc( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg1 [MAX_INPUT_LENGTH];

    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char(
	  "You do not have authorization to build in this zone.\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg1 );

    if ( arg1[0] == '\0' )
    {
	send_to_char(" Syntax:  rdesc clear\n\r",		ch );
	send_to_char("          rdesc add <text to add>\n\r",	ch );
	return;
    }

    if ( !str_cmp( arg1, "add" ) )
    {
	strcpy( buf, ch->in_room->description );

	if ( strlen( buf ) + strlen( argument ) >= MAX_STRING_LENGTH -4 )
	{
	    send_to_char("Room description too long, text not added.\n\r", ch );
	    return;
	}

	if ( buf[0] == '\0' )
	    strcpy( buf, argument );
	else
	    strcat( buf, argument );
	strcat( buf, "\n\r" );
	free_string( ch->in_room->description );
	ch->in_room->description = str_dup( buf );

	send_to_char("Text added to room description.\n\r", ch );
	return;
    }

    if ( !str_cmp( arg1, "clear" ) )
    {
	buf[0] = '\0';

	free_string( ch->in_room->description );
	ch->in_room->description = str_dup( buf );

	send_to_char("Room description cleared.\n\r", ch );
	return;
    }

    do_rdesc( ch, "" );

    return;
}


void do_rextra( CHAR_DATA *ch, char *argument  )
{
    char keyword[MAX_INPUT_LENGTH];
    char command[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    EXTRA_DESCR_DATA *ed;

    if ( IS_NPC( ch ) )
	return;

    if ( !can_build( ch, ch->in_room->area ) )
    {
	send_to_char(
	  "You do not have authorization to build in this zone.\n\r", ch );
	return;
    }

    argument = one_argument( argument, keyword );
    argument = one_argument( argument, command );

    if ( keyword[0] == '\0' || command[0] == '\0' )
    {
	send_to_char("Syntax:  rextra <keyword> <command> [text]\n\r", ch );
	send_to_char("\n\r", ch );
	send_to_char("Where command is one of:\n\r", ch );
	send_to_char(" clear create desc remove\n\r", ch );
	return;
    }

    for ( ed = ch->in_room->extra_descr; ed != NULL; ed = ed->next )
    {
	if ( is_name( keyword, ed->keyword ) )
	    break;
    }

    if ( !str_cmp( command, "create" ) )
    {
	if ( ed != NULL )
	{
	    send_to_char("Extra description already exists.\n\r", ch );
	    return;
	}

	if ( argument[0] == '\0' )
	{
    send_to_char("What should the extra description be called?\n\r", ch );
	    return;
	}

	ed = (EXTRA_DESCR_DATA *) alloc_perm( sizeof( *ed ) );

	ed->keyword = str_dup( argument );
	ed->description = str_dup( "\0" );

	ed->next = ch->in_room->extra_descr;
	ch->in_room->extra_descr = ed;

	send_to_char("Extra description created.\n\r", ch );
	return;
    }

    if ( ed == NULL )
    {
	send_to_char("No such extra description.\n\r", ch );
	return;
    }

    if ( !str_cmp( command, "clear" ) )
    {
	free_string( ed->description );
	ed->description = str_dup( "\0" );

	send_to_char("Extra description cleared.\n\r", ch );
	return;
    }

    if ( !str_cmp( command, "desc" ) )
    {
	strcpy( buf, ed->description );

	if ( strlen( buf ) + strlen( argument ) >= MAX_STRING_LENGTH -4 )
	{
	    send_to_char("Extra description too long, text not added.\n\r", ch );
	    return;
	}

	if ( buf[0] == '\0' )
	    strcpy( buf, argument );
	else
	    strcat( buf, argument );
	strcat( buf, "\n\r" );
	free_string( ed->description );
	ed->description = str_dup( buf );

	send_to_char("Text added to extra description.\n\r", ch );
	return;
    }

    if ( !str_cmp( command, "remove" ) )
    {
	EXTRA_DESCR_DATA *pextra;

	if ( ch->in_room->extra_descr == ed )
	{
	    ch->in_room->extra_descr = ed->next;
	}
	else
	{
	    for ( pextra = ch->in_room->extra_descr; pextra != NULL; pextra = pextra->next )
	    {
		if ( pextra->next == ed )
		    break;
	    }

	    pextra->next = ed->next;
	}

	free_extra_descr( ed );

	send_to_char("Extra description removed.\n\r", ch );
	return;
    }

    do_rextra( ch, "" );
    return;
}

